#include <cstdarg>
#include <cstdint>
#include <cstdlib>
#include <ostream>
#include <new>

extern "C" {

extern const uint32_t FIRST;

extern const uint32_t RENAMED;

void first();

void renamed();

} // extern "C"
