#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

typedef struct Bar Bar;

typedef struct {

} Foo;

extern const int32_t NUMBER;

extern Foo FOO;

extern const Bar BAR;

void root(void);
